;; nxml-menu.el --- All-in-one menu for nXML mode
;; $Id: nxml-menu.el,v 1.1 2003/10/31 12:45:13 smith Exp $
;; $Date: 2003/10/31 12:45:13 $
;;
;;; Commentary
;;
;; Adds a menu for nXML mode to your Emacs menubar.  The submenus
;; look like this:
;;
;;   nXML
;;     Validate
;;     Modify
;;     Move
;;     Markup
;;     Show
;;     --
;;     Customize nXML Options
;;
;; It also defines two new functions, `nxml-kill-element-forward' and
;; `nxml-kill-element-backward', and binds those to menu items and to
;; the key sequences "C-c C-k" and "C-c C-backspace".
;; 
;; To activate the menus, drop the nxml-menu.el file and accompanying
;; "nxml-manual" directory into the same directory where you have your
;; nxml-mode lisp files.  Then add the following to your .emacs
;;
;;   (require 'nxml-menu)
;;
;; Note: The included "nxml-manual" directory contains an HTML version
;;       of the nXML manual, generated from the texinfo version in the
;;       nXML distribution.

;;; Code

(require 'easymenu)

;; The following are used only for finding the generated HTML files
;; for the HTML version of the nXML manual.
(defvar nxml-menu-directory (file-name-directory load-file-name)
  "Directory that contains nxml-menu.el file."
  )
(defvar nxml-menu-base-uri (concat "file:///" nxml-menu-directory)
  "URI for directory that contains nxml-menu.el file."
  )

(defvar nxml-menu-definition-validate
  (list "Validate"
	["Toggle nXML Validate Mode"   rng-validate-mode t]
	["--" t]
	["Associate RNG Schema with Current Document Automatically"  rng-auto-set-schema t]
	["Associate RNG Schema with Current Document Manually" rng-set-schema-and-validate t]
	["Associate RNG Schema with Current Document By Type" rng-set-document-type-and-validate t]
	["---------" t]
	["Show RNG Schema Associated with Current Document" rng-what-schema t]
	["Save RNG Schema Association for Future Sessions"  rng-save-schema-location t]
	)
  "nXML Validate menu definition.")

(defvar nxml-menu-definition-modify
  (list "Modify"
	["Indent Region"   indent-region t]
	["Fill Paragraph"  fill-paragraph t]
	["Mark Paragraph"  nxml-mark-paragraph t]
	["------" t]
	["Kill Element Forward" nxml-kill-element-forward t]
	["Kill Element Backward"  nxml-kill-element-backward t]
	["-----" t]
	["Kill Tag Forward"  kill-sexp t]
	["Kill Tag Backward"  backward-kill-sexp t]
	)
  "nXML Modify menu definition.")

(defvar nxml-menu-definition-move
  (list "Move"
	["Forward Paragraph"    nxml-forward-paragraph t]
	["Backward Paragraph"   nxml-backward-paragraph t]
	["----" t]
	["Forward Element" nxml-forward-element t]
	["Backward Element"  nxml-backward-element t]
	["Down Through Current Element Tree"  nxml-down-element t]
	["Up Through Current Element Tree"   nxml-backward-up-element t]
	["------" t]
	["Forward Tag Beginning/End"   forward-sexp t]
	["Backward Tag Beginning/End"   backward-sexp t]
	["------" t]
	["First Error" rng-first-error t]
	["Next Error"  rng-next-error t]
	["Previous Error" rng-previous-error t]
	)
  "nXML Move menu definition.")

(defvar nxml-menu-definition-markup
  (list "Markup"
	["Complete in Current Context" nxml-complete t]
	["Insert Closing Tag (Block)" nxml-balanced-close-start-tag-block t]
	["Insert Closing Tag (Inline)" nxml-balanced-close-start-tag-inline t]
	["Finish Current Context" nxml-finish-element t]
	["----" t]
	["Apply Inferred Markup to Word Before Point" nxml-dynamic-markup-word t]
	["-----" t]
	["Insert Character Entity Reference by Full Name" nxml-insert-named-char t]
	["Insert Character by Full Name" (nxml-insert-named-char (universal-argument)) t]
	)
  "nXML Markup menu definition.")

(defvar nxml-menu-definition-show
  (list "Show"
	["Show this XML File in Web Browser"
	 (browse-url (concat "file:///" (buffer-file-name))) t]
	["Show Glyphs for Character Entity References"
	 nxml-toggle-char-ref-extra-display t]
	["-----" t]
	["Show nXML Manual (Info)" nxml-mode-visit-info-file t]
	(list "Show nXML Manual (HTML)"
	      ["Completion"
	       (browse-url (concat nxml-menu-base-uri "/nxml-manual/Completion.html")) t]
	      ["Inserting end-tags"
	       (browse-url (concat nxml-menu-base-uri "/nxml-manual/Inserting-end-tags.html")) t]
	      ["Paragraphs"
	       (browse-url (concat nxml-menu-base-uri "/nxml-manual/Paragraphs.html")) t]
	      (list "Locating a schema"
		    ["Intro"
		     (browse-url (concat nxml-menu-base-uri "/nxml-manual/Locating-a-schema.html")) t]
		    ["Commands for locating a schema"
		     (browse-url (concat nxml-menu-base-uri "/nxml-manual/Commands-for-locating-a-schema.html")) t]
	      (list "Schema locating files"
		    ["Intro"
		     (browse-url (concat nxml-menu-base-uri "/nxml-manual/Schema-locating-files.html")) t]
		    ["Schema locating file syntax basics"
		     (browse-url (concat nxml-menu-base-uri "/nxml-manual/Schema-locating-file-syntax-basics.html")) t]
		    ["Using the document's URI to locate a schema"
		     (browse-url (concat nxml-menu-base-uri "/nxml-manual/Using-the-document-s-URI-to-locate-a-schema.html")) t]
		    ["Using the document element to locate a schema"
		     (browse-url (concat nxml-menu-base-uri "/nxml-manual/Using-the-document-element-to-locate-a-schema.html")) t]
		    ["Using type identifiers in schema locating files"
		     (browse-url (concat nxml-menu-base-uri "/nxml-manual/Using-type-identifiers-in-schema-locating-files.html")) t]
		    ["Using multiple schema locating files"
		     (browse-url (concat nxml-menu-base-uri "/nxml-manual/Using-multiple-schema-locating-files.html")) t]
		    )
	      )
	      ["DTDs"
	       (browse-url (concat nxml-menu-base-uri "DTDs.html")) t]
	      ["Limitations"
	       (browse-url (concat nxml-menu-base-uri "Limitations.html")) t]
	      )
	["-------" t]
	["Show nXML Version" nxml-version t]
	)
  "nXML Show menu definition.")

(defvar nxml-menu-definition-nxml
  (list "nXML"
	nxml-menu-definition-validate
	nxml-menu-definition-modify
	nxml-menu-definition-move
	nxml-menu-definition-markup
	nxml-menu-definition-show
	["----" t]
	["Customize nXML Options" (customize-group 'nxml) t]
	)
  "nXML nXML menu definition.")

(add-hook 'nxml-mode-hook
	  (lambda () 
	    (easy-menu-define 
	      nxml-menu-nxml
	      nxml-mode-map
	      "nXML"
	      nxml-menu-definition-nxml)
	    )
	  )

(defun nxml-mode-visit-info-file ()
  (interactive)
  "Visit the nxml info file."
  (require 'info)
  (Info-find-node "nxml-mode.info" "Top")
  )

(defun nxml-kill-element-forward ()
  (interactive)
  "Kill the element (and all its child content) after point."
  (let ((beg (point))) (nxml-forward-element) (delete-region beg (point)))
  )

(defun nxml-kill-element-backward ()
  (interactive)
  "Kill the element (and all its child content) before point."
  (let ((beg (point))) (nxml-backward-element) (delete-region beg (point)))
  )

(defun nxml-mode-kill-element-keys ()
  "Defines bindings for nxml kill-element functions."
  (define-key nxml-mode-map "\C-c\C-k" 'nxml-kill-element-forward)
  (define-key nxml-mode-map [?\C-cC-backspace] 'nxml-kill-element-backward)
  )

(add-hook 'nxml-mode-hook 'nxml-mode-kill-element-keys)

(provide 'nxml-menu)

;;; nxml-menu.el ends here
